import java.io.*;
import java.net.*;
import java.awt.*;

/**
 * CharChat -- A chat program between a server and one client.
 *
 * To run as a client, name the server as the first command line 
 * argument:
 *   java SingleChat bluehorse.com
 * or
 *   java SingleChat local          // to run locally
 */

public class CharChat {
    private int                port = 5001;    // The default port.

    public static void main(String args[]) {

       String s = null;
       if (args.length > 0)
          s = args[0];

       CharChat chat = new CharChat();
       if (s == null)
          chat.server();
       else
          chat.client(s);        
    }
   
    // As a server, we create a server socket bound to the specified
    // port, wait for a connection, and then spawn a thread to 
    // read data coming in over the network via the socket.

    private void server() {
        Socket sock = null;
        ServerSocket serverSock = null;

        try {
           serverSock = new ServerSocket(port, 1);

           //We'll only accept one connection for this server.
           sock = serverSock.accept();

           new CharChatSend(sock).start();
           new CharChatReceive(sock).start();
       
        } catch (IOException x) {
           System.out.println(x.getMessage() +
               ": Failed to connect to client.");
           System.exit(1);        

        } finally {

           // At this point, we don't need the ServerSocket anymore.
           if (serverSock != null) {
               try {
                   serverSock.close();
               } catch (IOException x) {
               }
           }
        }        

    }

    // As a client, we create a socket bound to the specified port,
    // connect to the specified host, and then spawn a thread to 
    // read data coming coming in over the network via the socket.

    private void client(String serverName) {
        try {
            if (serverName.equals("local"))
               serverName = null;

            InetAddress serverAddr = InetAddress.getByName(serverName);
            Socket sock = 
               new Socket(serverAddr.getHostName(), port, true);           

            new CharChatSend(sock).start();
            new CharChatReceive(sock).start();

        } catch (IOException e) {
            System.out.println(e.getMessage() +
               ": Failed to connect to server.");
        }
    }

}


/*
 * CharChatReceive takes data sent on a socket and displays it in
 * the standard output.
 */
class CharChatReceive extends Thread {
    private Socket sock;
    private DataInputStream remoteIn;
    private boolean listening = true;

    public CharChatReceive(Socket sock) throws IOException {
        this.sock = sock;
        remoteIn = new DataInputStream(sock.getInputStream());
    }

    public synchronized void run() {
        String s;
        try {

            while (listening) {
                s = remoteIn.readUTF();   
                System.out.println(s);    
            }

        } catch (IOException e) {
            System.out.println(e.getMessage() +
               ": Connection to peer lost.");

        } finally {
           try {
              if (remoteIn != null) {
                 remoteIn.close();
                 remoteIn = null;
              }

              if (sock != null) {
                 sock.close();
                 sock = null;
              }

           } catch (IOException x) {
           }
        }
    }

    protected void finalize() throws Throwable {
        try {
           if (remoteIn != null)
              remoteIn.close();
           if (sock != null)
              sock.close();
        } catch (IOException x) {
        }
        super.finalize();
    }

}

/*
 * CharChatSend takes data entered on the standard input and
 * sends it out over a socket.
 */
class CharChatSend extends Thread {
    private Socket sock;
    private DataOutputStream remoteOut;
    private boolean listening = true;

    public CharChatSend(Socket sock) throws IOException {
        this.sock = sock;
        remoteOut = new DataOutputStream(sock.getOutputStream());
    }

    public synchronized void run() {
        DataInputStream in = null;
        String s;
        try {

            in = new DataInputStream(System.in);
            while (listening) {
                s = in.readLine();
                if (s.equals(""))
                   break;   
                remoteOut.writeUTF(s);    
            }

        } catch (IOException e) {
            System.out.println(e.getMessage() +
               ": Connection to peer lost.");

        } finally {
           try {
              if (in != null) 
                 in.close();

              if (remoteOut != null) {
                 remoteOut.close();
                 remoteOut = null;
              }

              // Socket closed in receive thread

           } catch (IOException x) {
           }
        }
    }

    protected void finalize() throws Throwable {
        try {
           if (remoteOut != null)
              remoteOut.close();

           // Socket closed in Recive thread

        } catch (IOException x) {
        }
        super.finalize();
    }

}
